? base/perlxsi.c
Index: base/utils.c
===================================================================
RCS file: /cvsroot/nagios/nagios/base/utils.c,v
retrieving revision 1.216
diff -u -r1.216 utils.c
--- base/utils.c	13 Dec 2007 17:14:28 -0000	1.216
+++ base/utils.c	14 Jan 2008 05:15:46 -0000
@@ -2813,7 +2813,7 @@
 	        }
 
 	/* open the command file for reading (non-blocked) - O_TRUNC flag cannot be used due to errors on some systems */
-	if((command_file_fd=open(command_file,O_RDONLY | O_NONBLOCK))<0){
+	if((command_file_fd=open(command_file,O_RDWR | O_NONBLOCK))<0){
 
 		logit(NSLOG_RUNTIME_ERROR,TRUE,"Error: Could not open external command file for reading via open(): (%d) -> %s\n",errno,strerror(errno));
 
@@ -3718,6 +3718,8 @@
 /* worker thread - artificially increases buffer of named pipe */
 void * command_file_worker_thread(void *arg){
 	char input_buffer[MAX_EXTERNAL_COMMAND_LENGTH];
+ 	struct pollfd pfd;
+ 	int pollval;
 	struct timeval tv;
 	int buffer_items=0;
 	int result=0;
@@ -3734,14 +3736,45 @@
 		/* should we shutdown? */
 		pthread_testcancel();
 
-		/**** POLL() AND SELECT() DON'T SEEM TO WORK ****/
-		/* wait a bit */
-		tv.tv_sec=0;
-		tv.tv_usec=500000;
-		select(0,NULL,NULL,NULL,&tv);
-
-		/* should we shutdown? */
-		pthread_testcancel();
+		/* wait for data to arrive */
+		/* select seems to not work, so we have to use poll instead */
+		pfd.fd=command_file_fd;
+		pfd.events=POLLIN;
+		pollval=poll(&pfd,1,500);
+
+ 		/* loop if no data */
+ 		if(pollval==0)
+ 			continue;
+
+ 		/* check for errors */
+ 		if(pollval==-1){
+ 
+ 			switch(errno){
+ 			case EBADF:
+ 				write_to_log("command_file_worker_thread(): poll(): EBADF",logging_options,NULL);
+ 				break;
+ 			case ENOMEM:
+ 				write_to_log("command_file_worker_thread(): poll(): ENOMEM",logging_options,NULL);
+ 				break;
+ 			case EFAULT:
+ 				write_to_log("command_file_worker_thread(): poll(): EFAULT",logging_options,NULL);
+ 				break;
+ 			case EINTR:
+ 				/* this can happen when running under a debugger like gdb */
+ 				/*
+ 				write_to_log("command_file_worker_thread(): poll(): EINTR (impossible)",logging_options,NULL);
+ 				*/
+ 				break;
+ 			default:
+ 				write_to_log("command_file_worker_thread(): poll(): Unknown errno value.",logging_options,NULL);
+ 				break;
+ 			        }
+ 
+ 			continue;
+ 		        }
+ 
+ 		/* should we shutdown? */
+ 		pthread_testcancel();
 
 		/* get number of items in the buffer */
 		pthread_mutex_lock(&external_command_buffer.buffer_lock);
@@ -3756,6 +3789,7 @@
 		if(buffer_items<external_command_buffer_slots){
 
 			/* clear EOF condition from prior run (FreeBSD fix) */
+			/* FIXME: use_poll_on_cmd_pipe: Still needed? */
 			clearerr(command_file_fp);
 
 			/* read and process the next command in the file */
